@@ -1,18 +1,23 @@
 """
 pakbase module
-  This module contains the base package classs from which
+  This module contains the base package class from which
   all of the other packages inherit from.
 
 """
 
 from __future__ import print_function
-import numpy as np
-from .utils import Util2d, Util3d, Transient2d, MfList, check
-from numpy.lib.recfunctions import stack_arrays
+
 import os
-from .modflow.mfparbc import ModflowParBc as mfparbc
+import sys
+import platform
 import webbrowser as wb
 
+import numpy as np
+from numpy.lib.recfunctions import stack_arrays
+
+from .modflow.mfparbc import ModflowParBc as mfparbc
+from .utils import Util2d, Util3d, Transient2d, MfList, check
+
 
 class Package(object):
     """
@@ -21,7 +26,7 @@ class Package(object):
     """
 
     def __init__(self, parent, extension='glo', name='GLOBAL', unit_number=1,
-                 extra='', allowDuplicates=False):
+                 extra='', filenames=None, allowDuplicates=False):
         """
         Package init
 
@@ -31,9 +36,15 @@ class Package(object):
             extension = [extension]
         self.extension = []
         self.file_name = []
-        for e in extension:
+        for idx, e in enumerate(extension):
             self.extension.append(e)
             file_name = self.parent.name + '.' + e
+            if filenames is not None:
+                try:
+                    if filenames[idx] is not None:
+                        file_name = filenames[idx]
+                except:
+                    pass
             self.file_name.append(file_name)
         self.fn_path = os.path.join(self.parent.model_ws, self.file_name[0])
         if (not isinstance(name, list)):
@@ -63,7 +74,7 @@ class Package(object):
                     else:
                         s = s + ' {0:s} (list, items = {1:d}\n'.format(attr,
                                                                        len(
-                                                                               value))
+                                                                           value))
                 elif (isinstance(value, np.ndarray)):
                     s = s + ' {0:s} (array, shape = {1:s})\n'.format(attr,
                                                                      value.shape.__str__()[
@@ -80,17 +91,17 @@ class Package(object):
             if isinstance(item, MfList):
                 if not isinstance(item, list) and not isinstance(item, tuple):
                     assert item in list(
-                            self.stress_period_data.data.keys()), "package.__getitem__() kper " + str(
-                            item) + " not in data.keys()"
+                        self.stress_period_data.data.keys()), "package.__getitem__() kper " + str(
+                        item) + " not in data.keys()"
                     return self.stress_period_data[item]
                 else:
                     if item[1] not in self.dtype.names:
                         raise Exception(
-                                "package.__getitem(): item \'" + item + "\' not in dtype names " + str(
-                                        self.dtype.names))
+                            "package.__getitem(): item \'" + item + "\' not in dtype names " + str(
+                                self.dtype.names))
                     assert item[0] in list(
-                            self.stress_period_data.data.keys()), "package.__getitem__() kper " + str(
-                            item[0]) + " not in data.keys()"
+                        self.stress_period_data.data.keys()), "package.__getitem__() kper " + str(
+                        item[0]) + " not in data.keys()"
                     if self.stress_period_data.vtype[item[0]] == np.recarray:
                         return self.stress_period_data[item[0]][item[1]]
 
@@ -205,7 +216,7 @@ class Package(object):
                 if isinstance(self.stress_period_data.data[per], np.recarray):
                     spd = self.stress_period_data.data[per]
                     inds = (spd.k, spd.i, spd.j) if self.parent.structured \
-                            else (spd.node)
+                        else (spd.node)
 
                     # General BC checks
                     # check for valid cell indices
@@ -225,7 +236,8 @@ class Package(object):
                             # also checks for nan values
                             elev_name = chk.bc_stage_names[self.name[0]]
                             botms = self.parent.dis.botm.array[inds]
-                            chk.stress_period_data_values(spd, spd[elev_name] < botms,
+                            chk.stress_period_data_values(spd, spd[
+                                elev_name] < botms,
                                                           col=elev_name,
                                                           error_name='BC elevation below cell bottom',
                                                           error_type='Error')
@@ -240,8 +252,12 @@ class Package(object):
 
             # check for confined layers above convertable layers
             confined = False
+            thickstrt = False
+            for option in self.options:
+                if option.lower() == 'thickstrt':
+                    thickstrt = True
             for i, l in enumerate(self.laytyp.array.tolist()):
-                if l == 0 or l < 0 and self.thickstrt:
+                if l == 0 or l < 0 and thickstrt:
                     confined = True
                     continue
                 if confined and l > 0:
@@ -305,7 +321,8 @@ class Package(object):
                                  chk.property_threshold_values['vkcb'],
                                  'quasi-3D confining bed Kv')
 
-            if not np.all(self.parent.dis.steady):  # only check storage if model is transient
+            if not np.all(
+                    self.parent.dis.steady):  # only check storage if model is transient
 
                 # do the same for storage if the model is transient
                 sarrays = {'ss': self.ss.array, 'sy': self.sy.array}
@@ -313,8 +330,10 @@ class Package(object):
                     chk._add_to_summary(type='Warning',
                                         desc='\r    STORAGECOEFFICIENT option is activated, \
                                               storage values are read storage coefficients')
-                    sarrays['ss'] /= self.parent.dis.thickness.array
-                    sarrays['sy'] /= self.parent.dis.thickness.array
+                    tshape = (self.parent.nlay, self.parent.nrow,
+                              self.parent.ncol)
+                    sarrays['ss'].shape != tshape
+                    sarrays['sy'].shape != tshape
 
                 chk.values(sarrays['ss'], active & (sarrays['ss'] < 0),
                            'zero or negative specific storage values', 'Error')
@@ -324,8 +343,8 @@ class Package(object):
 
                 # only check specific yield for convertible layers
                 inds = np.array(
-                        [True if l > 0 or l < 0 and 'THICKSRT' in self.options
-                         else False for l in self.laytyp])
+                    [True if l > 0 or l < 0 and 'THICKSRT' in self.options
+                     else False for l in self.laytyp])
                 sarrays['sy'] = sarrays['sy'][inds, :, :]
                 active = active[inds, :, :]
                 chk.values(sarrays['sy'], active & (sarrays['sy'] < 0),
@@ -337,7 +356,7 @@ class Package(object):
 
         else:
             txt = 'check method not implemented for {} Package.'.format(
-                    self.name[0])
+                self.name[0])
             if f is not None:
                 if isinstance(f, str):
                     pth = os.path.join(self.parent.model_ws, f)
@@ -360,8 +379,8 @@ class Package(object):
                                                                        'column',
                                                                        name[
                                                                            k].lower().replace(
-                                                                               ' layer ',
-                                                                               ''))
+                                                                           ' layer ',
+                                                                           ''))
                 txt += '    {:10d}{:10d}{:10d}{:15.7g}\n'.format(k + 1, i + 1,
                                                                  j + 1,
                                                                  v[k, i, j])
@@ -369,7 +388,7 @@ class Package(object):
             txt += '    {:>10s}{:>10s}{:>15s}\n'.format('row', 'column',
                                                         name[
                                                             0].lower().replace(
-                                                                ' layer ', ''))
+                                                            ' layer ', ''))
             for [i, j] in idx:
                 txt += '    {:10d}{:10d}{:15.7g}\n'.format(i + 1, j + 1,
                                                            v[i, j])
@@ -465,15 +484,15 @@ class Package(object):
             if isinstance(value, MfList):
                 if self.parent.verbose:
                     print('plotting {} package MfList instance: {}'.format(
-                            self.name[0], item))
+                        self.name[0], item))
                 if key is None:
                     names = ['{} location stress period {} layer {}'.format(
-                            self.name[0], kper + 1, k + 1)
+                        self.name[0], kper + 1, k + 1)
                              for k in range(self.parent.nlay)]
                     colorbar = False
                 else:
                     names = ['{} {} data stress period {} layer {}'.format(
-                            self.name[0], key, kper + 1, k + 1)
+                        self.name[0], key, kper + 1, k + 1)
                              for k in range(self.parent.nlay)]
                     colorbar = True
 
@@ -488,50 +507,50 @@ class Package(object):
             elif isinstance(value, Util3d):
                 if self.parent.verbose:
                     print('plotting {} package Util3d instance: {}'.format(
-                            self.name[0], item))
+                        self.name[0], item))
                 # fignum = list(range(ifig, ifig + inc))
                 fignum = list(range(ifig, ifig + value.shape[0]))
                 ifig = fignum[-1] + 1
                 caxs.append(
-                        value.plot(filename_base=fileb, file_extension=fext,
-                                   mflay=mflay,
-                                   fignum=fignum, colorbar=True))
+                    value.plot(filename_base=fileb, file_extension=fext,
+                               mflay=mflay,
+                               fignum=fignum, colorbar=True))
             elif isinstance(value, Util2d):
                 if len(value.shape) == 2:
                     if self.parent.verbose:
                         print('plotting {} package Util2d instance: {}'.format(
-                                self.name[0], item))
+                            self.name[0], item))
                     fignum = list(range(ifig, ifig + 1))
                     ifig = fignum[-1] + 1
                     caxs.append(
-                            value.plot(filename_base=fileb,
-                                       file_extension=fext,
-                                       fignum=fignum, colorbar=True))
+                        value.plot(filename_base=fileb,
+                                   file_extension=fext,
+                                   fignum=fignum, colorbar=True))
             elif isinstance(value, Transient2d):
                 if self.parent.verbose:
                     print(
-                            'plotting {} package Transient2d instance: {}'.format(
-                                    self.name[0], item))
+                        'plotting {} package Transient2d instance: {}'.format(
+                            self.name[0], item))
                 fignum = list(range(ifig, ifig + inc))
                 ifig = fignum[-1] + 1
                 caxs.append(
-                        value.plot(filename_base=fileb, file_extension=fext,
-                                   kper=kper,
-                                   fignum=fignum, colorbar=True))
+                    value.plot(filename_base=fileb, file_extension=fext,
+                               kper=kper,
+                               fignum=fignum, colorbar=True))
             elif isinstance(value, list):
                 for v in value:
                     if isinstance(v, Util3d):
                         if self.parent.verbose:
                             print(
-                                    'plotting {} package Util3d instance: {}'.format(
-                                            self.name[0], item))
+                                'plotting {} package Util3d instance: {}'.format(
+                                    self.name[0], item))
                         fignum = list(range(ifig, ifig + inc))
                         ifig = fignum[-1] + 1
                         caxs.append(
-                                v.plot(filename_base=fileb,
-                                       file_extension=fext,
-                                       mflay=mflay,
-                                       fignum=fignum, colorbar=True))
+                            v.plot(filename_base=fileb,
+                                   file_extension=fext,
+                                   mflay=mflay,
+                                   fignum=fignum, colorbar=True))
             else:
                 pass
 
@@ -582,13 +601,13 @@ class Package(object):
     def webdoc(self):
         if self.parent.version == 'mf2k':
             wb.open(
-                    'http://water.usgs.gov/nrp/gwsoftware/modflow2000/Guide/' + self.url)
+                'http://water.usgs.gov/nrp/gwsoftware/modflow2000/Guide/' + self.url)
         elif self.parent.version == 'mf2005':
             wb.open(
-                    'http://water.usgs.gov/ogw/modflow/MODFLOW-2005-Guide/' + self.url)
+                'http://water.usgs.gov/ogw/modflow/MODFLOW-2005-Guide/' + self.url)
         elif self.parent.version == 'ModflowNwt':
             wb.open(
-                    'http://water.usgs.gov/ogw/modflow-nwt/MODFLOW-NWT-Guide/' + self.url)
+                'http://water.usgs.gov/ogw/modflow-nwt/MODFLOW-NWT-Guide/' + self.url)
 
     def write_file(self, check=False):
         """
@@ -599,7 +618,8 @@ class Package(object):
         return
 
     @staticmethod
-    def load(model, pack_type, f, nper=None, pop_key_list=None, check=True):
+    def load(model, pack_type, f, nper=None, pop_key_list=None, check=True,
+             unitnumber=None, ext_unit_dict=None):
         """
         The load method has not been implemented for this package.
 
@@ -609,7 +629,12 @@ class Package(object):
 
         if not hasattr(f, 'read'):
             filename = f
-            f = open(filename, 'r')
+            if platform.system().lower() == 'windows' and \
+                            sys.version_info[0] < 3:
+                import io
+                f = io.open(filename, 'r')
+            else:
+                f = open(filename, 'r')
         # dataset 0 -- header
         while True:
             line = f.readline()
@@ -632,9 +657,7 @@ class Package(object):
         t = line.strip().split()
         ipakcb = 0
         try:
-            if int(t[1]) != 0:
-                ipakcb = 53
-                pop_key_list = model.pop_key_list(int(t[1]), pop_key_list)
+            ipakcb = int(t[1])
         except:
             pass
         options = []
@@ -654,15 +677,16 @@ class Package(object):
         # set partype
         #  and read phiramp for modflow-nwt well package
         partype = ['cond']
-        if "modflowwel" in str(pack_type).lower():
+        if 'modflowwel' in str(pack_type).lower():
             partype = ['flux']
 
-        if "nwt" in model.version.lower() and 'flopy.modflow.mfwel.modflowwel'.lower() in str(pack_type).lower():
+        if 'nwt' in model.version.lower() and \
+            'flopy.modflow.mfwel.modflowwel'.lower() in str(pack_type).lower():
 
             specify = False
             ipos = f.tell()
             line = f.readline()
-            # test for specify keyword if a NWT well file - This is a temporary hack
+            # test for specify keyword if a NWT well file
             if 'specify' in line.lower():
                 specify = True
                 t = line.strip().split()
@@ -694,9 +718,9 @@ class Package(object):
         for iper in range(nper):
             if model.verbose:
                 print(
-                        "   loading " + str(
-                            pack_type) + " for kper {0:5d}".format(
-                                iper + 1))
+                    "   loading " + str(
+                        pack_type) + " for kper {0:5d}".format(
+                        iper + 1))
             line = f.readline()
             if line == '':
                 break
@@ -718,6 +742,9 @@ class Package(object):
                 for ibnd in range(itmp):
                     line = f.readline()
                     if "open/close" in line.lower():
+                        binary = False
+                        if '(binary)' in line.lower():
+                            binary = True
                         # need to strip out existing path seps and
                         # replace current-system path seps
                         raw = line.strip().split()
@@ -731,22 +758,39 @@ class Package(object):
                         fname = os.path.join(*raw)
                         oc_filename = os.path.join(model.model_ws, fname)
                         assert os.path.exists(
-                                oc_filename), "Package.load() error: open/close filename " + \
-                                              oc_filename + " not found"
+                            oc_filename), "Package.load() error: open/close filename " + \
+                                          oc_filename + " not found"
                         try:
-                            current = np.genfromtxt(oc_filename,
-                                                    dtype=current.dtype)
+                            if binary:
+                                dtype2 = []
+                                for name in current.dtype.names:
+                                    dtype2.append((name, np.float32))
+                                dtype2 = np.dtype(dtype2)
+                                d = np.fromfile(oc_filename,
+                                                dtype=dtype2,
+                                                count=itmp)
+                                current = np.array(d, dtype=current.dtype)
+                            else:
+                                #current = np.genfromtxt(oc_filename,
+                                #                         dtype=current.dtype)
+                                #if len(current.shape) == 1:
+                                cd = current.dtype
+                                current = np.loadtxt(oc_filename).transpose()
+                                if current.ndim == 1:
+                                    current = np.atleast_2d(current).transpose()
+                                #current = np.atleast_2d(np.loadtxt(oc_filename,
+                                #                                   dtype=current.dtype)).transpose()
+                                current = np.core.records.fromarrays(current,dtype=cd)
                             current = current.view(np.recarray)
                         except Exception as e:
                             raise Exception(
-                                    "Package.load() error loading open/close file " + oc_filename + \
-                                    " :" + str(e))
+                                "Package.load() error loading open/close file " + oc_filename + \
+                                " :" + str(e))
                         assert current.shape[
                                    0] == itmp, "Package.load() error: open/close rec array from file " + \
-                                               oc_filename + " shape (" + str(
-                                current.shape) + \
+                                               oc_filename + " shape (" + str(current.shape) + \
                                                ") does not match itmp: {0:d}".format(
-                                                       itmp)
+                                                   itmp)
                         break
                     try:
                         t = line.strip().split()
@@ -825,11 +869,25 @@ class Package(object):
             else:
                 stress_period_data[iper] = bnd_output
 
+        dtype = pack_type.get_empty(0, aux_names=aux_names,
+                                    structured=model.structured).dtype
+
+        # set package unit number
+        unitnumber = None
+        filenames = [None, None]
+        if ext_unit_dict is not None:
+            unitnumber, filenames[0] = \
+                model.get_ext_dict_attr(ext_unit_dict,
+                                        filetype=pack_type.ftype())
+            if ipakcb > 0:
+                iu, filenames[1] = \
+                    model.get_ext_dict_attr(ext_unit_dict, unit=ipakcb)
+                model.add_pop_key_list(ipakcb)
+
         pak = pack_type(model, ipakcb=ipakcb,
-                        stress_period_data=stress_period_data, \
-                        dtype=pack_type.get_empty(0, aux_names=aux_names,
-                                                  structured=model.structured).dtype, \
-                        options=options)
+                        stress_period_data=stress_period_data,
+                        dtype=dtype, options=options,
+                        unitnumber=unitnumber, filenames=filenames)
         if check:
             pak.check(f='{}.chk'.format(pak.name[0]),
                       verbose=pak.parent.verbose, level=0)
